from pyjsonrpc import JsonRpc, rpcmethod
from modbus_tk.defines import (READ_COILS, READ_DISCRETE_INPUTS,
                               READ_HOLDING_REGISTERS, READ_INPUT_REGISTERS,
                               WRITE_SINGLE_COIL, WRITE_SINGLE_REGISTER,
                               WRITE_MULTIPLE_COILS, WRITE_MULTIPLE_REGISTERS)


class Dispatcher(JsonRpc):

    def __init__(self, modbus_master):
        self.modbus_master = modbus_master

        super(JsonRpc, self).__init__()

    @rpcmethod
    def read_coils(self, starting_address, quantity, slave_id=1):
        """ Execute Modbus function code 01: read status of coils.

        :param starting_address: Number of starting address.
        :param quantity: Number of coils to read.
        :param slave: Number with Slave id, default 1.
        """
        return self.modbus_master.execute(int(slave_id), READ_COILS,
                                          int(starting_address),
                                          int(quantity))

    @rpcmethod
    def read_discrete_inputs(self, starting_address, quantity,
                             slave_id=1):
        """ Execute Modbus function code 02: read status of discreate inputs.

        :param starting_address: Number of starting address.
        :param quantity: Number of discrete inputs to read.
        :param slave: Number with Slave id, default 1.
        """
        return self.modbus_master.execute(int(slave_id), READ_DISCRETE_INPUTS,
                                          int(starting_address),
                                          int(quantity))

    @rpcmethod
    def read_holding_registers(self, starting_address, quantity,
                               slave_id=1):
        """ Execute Modbus function code 03: read contents of contiguous block
        of holding registers.

        :param starting_address: Number of starting address.
        :param quantity: Number of holding registers to read.
        :param slave: Number with Slave id, default 1.
        """
        return self.modbus_master.execute(int(slave_id),
                                          READ_HOLDING_REGISTERS,
                                          int(starting_address), int(quantity))

    @rpcmethod
    def read_input_registers(self, starting_address, quantity,
                             slave_id=1):
        """ Execute Modbus function code 04: read contents of contiguous block
        of input registers.

        :param starting_address: Number of starting address.
        :param quantity: Number of input registers to read.
        :param slave: Number with Slave id, default 1.
        """
        return self.modbus_master.execute(int(slave_id),
                                          READ_INPUT_REGISTERS,
                                          int(starting_address), int(quantity))

    @rpcmethod
    def write_single_coil(self, address, value, slave_id=1):
        """ Execute Modbus function code 05: write value to single coil.

        :param address: Address of coil.
        :param value: Value to write to coil.
        :param slave: Number with Slave id, default 1.
        """
        return self.modbus_master.execute(int(slave_id),
                                          WRITE_SINGLE_COIL, int(address),
                                          output_value=int(value))

    @rpcmethod
    def write_single_register(self, address, value, slave_id=1):
        """ Execute Modbus function code 06: write value to single holding
        register.

        :param address: Address of holding register.
        :param value: Value to write to holding register.
        :param slave: Number with Slave id, default 1.
        """
        return self.modbus_master.execute(int(slave_id),
                                          WRITE_SINGLE_REGISTER, int(address),
                                          output_value=int(value))

    @rpcmethod
    def write_multiple_coils(self, starting_address, values,
                             slave_id=1):
        """ Execute Modbus function code 15: write sequence of values to a
        contiguous block of coils.

        .. note:: This method doesn't follow the interface as described in the
            Modbus specification. The specification describes 2 extra
            'parameters': quantity of outputs and byte count. Tolk will derive
            them from paramater `values`.

        :param starting_address: Number of starting address.
        :param values: List with values.
        :param slave: Number with Slave id, default 1.
        """
        values = [int(v) for v in values]
        return self.modbus_master.execute(int(slave_id),
                                          WRITE_MULTIPLE_COILS,
                                          int(starting_address),
                                          output_value=values)

    @rpcmethod
    def write_multiple_registers(self, starting_address, values,
                                 slave_id=1):
        """ Execute Modbus function code 16: write sequence of values to a
        contiguous block of holding registers.

        .. note:: This method doesn't follow the interface as described in the
            Modbus specification. The specification describes 2 extra
            'parameters': quantity of outputs and byte count. Tolk will derive
            them from paramater `values`.

        :param starting_address: Number of starting address.
        :param values: List with values to write.
        :param slave: Number with Slave id, default 1.
        """
        values = [int(v) for v in values]
        return self.modbus_master.execute(int(slave_id),
                                          WRITE_MULTIPLE_REGISTERS,
                                          int(starting_address),
                                          output_value=values)



#!/usr/bin/env python
""" Send JSON-RPC requests.

Usage:
    json_rpc_client read_coils <starting-address> <quantity> [--slave-id=<nr> --socket=<path>]
    json_rpc_client read_discrete_inputs <starting-address> <quantity> [--slave-id=<nr> --socket=<path>]
    json_rpc_client read_holding_registers <starting-address> <quantity> [--slave-id=<nr> --socket=<path>]
    json_rpc_client read_input_registers <starting-address> <quantity> [--slave-id=<nr> --socket=<path>]
    json_rpc_client write_single_coil <address> <value> [--slave-id=<nr> --socket=<path>]
    json_rpc_client write_single_register <address> <value> [--slave-id=<nr> --socket=<path>]
    json_rpc_client write_multiple_coils <starting_address> <values> [--slave-id=<nr> --socket=<path>]
    json_rpc_client write_multiple_registers <starting_address> <values> [--slave-id=<nr> --socket=<path>]

Options:
    -h --help                   Show this screen.
    -s --slave-id=<nr>          Id of slave [default: 1].
    --socket=</tmp/tolk.sock>   Location of Tolk's socket [default: /tmp/tolk.sock].

"""
import json
import socket
from uuid import uuid4
from docopt import docopt
from collections import namedtuple

READ = 0
SINGLE_WRITE = 1
MULTIPLE_WRITE = 2

Method = namedtuple('Method', ['name', 'type_'])


def main():
    args = docopt(__doc__)

    s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    s.connect('/tmp/tolk.sock')

    if args['read_coils']:
        method = Method(name='read_coils',  type_=READ)
    elif args['read_discrete_inputs']:
        method = Method(name='discrete_inputs',  type_=READ)
    elif args['read_holding_registers']:
        method = Method(name='read_holding_registers', type_=READ)
    elif args['read_input_registers']:
        method = Method(name='read_input_registers', type_=READ)
    elif args['write_single_coil']:
        method = Method(name='write_single_coil', type_=SINGLE_WRITE)
    elif args['write_single_register']:
        method = Method(name='write_single_register', type_=SINGLE_WRITE)
    elif args['write_multiple_coils']:
        method = Method(name='write_multiple_coils', type_=MULTIPLE_WRITE)
    elif args['write_multiple_registers']:
        method = Method(name='write_multiple_registers', type_=MULTIPLE_WRITE)

    if method.type_ == READ:
        params = {
            'starting_address': int(args['<starting-address>']),
            'quantity': int(args['<quantity>']),
        }
    elif method.type_ == SINGLE_WRITE:
        params = {
            'address': int(args['<address>']),
            'value': int(args['<value>']),
        }
    elif method.type_ == MULTIPLE_WRITE:
        # Transform a string like '1,0,13' into a list of integers.
        values = [int(v) for v in args['<values>'].split(',')]

        params = {
            'starting_address': int(args['<starting_address>']),
            'values': values,
        }

    params['slave_id'] = int(args['--slave-id'])

    msg = get_json_rpc_message(method.name, params)
    s.sendall(msg)

    resp = json.loads(s.recv(1024))
    s.close()

    print(json.dumps(resp, sort_keys=True, indent=4, separators=(',', ': ')))


def get_json_rpc_message(method, params):
    """ Return a JSON-RPC formatted string.

    :param method: Name of JSON-RPC method.
    :param params: Dictionary with JSON-RPC parameters.
    :return: JSON-RPC valid string.
    """
    return json.dumps({
        'jsonrpc': '2.0',
        'method': method,
        'params': params,
        'id': uuid4().int,
    })

if __name__ == '__main__':
    main()



#!/usr/bin/env python
""" Modbus slave listening at a port.

Usage:
    modbus_tcp_slave.py [--host=<name> --port=<nr>]

Options:
    -h --help       Show this screen.
    --host=<name>   Name of host [default: ""].
    --port=<nr>     The port where slave is listening at [default: 502].

"""
import sys
import time
from docopt import docopt

from logbook import Logger, StreamHandler
from modbus_tk.utils import create_logger
import modbus_tk.defines as cst
from modbus_tk.modbus_tcp import TcpServer

StreamHandler(sys.stdout).push_application()
log = Logger(__name__)

logger = create_logger(name="console", record_format="%(message)s")


def main():
    args = docopt(__doc__)
    server = TcpServer(port=int(args['--port']))
    slave = server.add_slave(1)

    slave.add_block(0, cst.ANALOG_INPUTS, 0, 100)
    log.info('Add analog inputs data block from register 0 to 99 on slave 1')
    slave.add_block(1, cst.DISCRETE_INPUTS, 0, 100)
    log.info('Add discrete inputs data block from register 0 to 99 on slave 1')
    slave.add_block(2, cst.COILS, 100, 100)
    log.info('Add coils data block from register 100 to 199 on slave 1')
    slave.add_block(3, cst.HOLDING_REGISTERS, 100, 100)
    log.info('Add holding registers block from register 100 to 199 on slave 1')

    server.start()
    log.info('TcpServer started listening at port {0}.'.format(args['--port']))

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        log.info('Received SIGINT. Exiting')
    finally:
        server.stop()
        log.info('TcpServer has stopped.')


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Tolk
====

Tolk is JSON-RPC proxy for doing Modbus communication over RTU and TCP.
"""
from setuptools import setup

setup(name='Tolk',
      version='0.1.4',
      author='Auke Willem Oosterhoff',
      author_email='oosterhoff@baopt.nl',
      description='JSON-RPC proxy for talking Modbus over RTU and TCP.',
      url='https://github.com/AdvancedClimateSystems/Tolk/',
      long_description=__doc__,
      license='MPL',
      packages=[
          'tolk',
          'scripts',
      ],
      install_requires=[
          'docopt>=0.6.2',
          'logbook>=0.11.2',
          'modbus-tk>=0.4.3',
          'python-jsonrpc>=0.7.12',
      ],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
          'Operating System :: Unix',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Topic :: Software Development :: Embedded Systems',
      ],
      entry_points={
          'console_scripts': [
            'json_rpc_client = scripts.json_rpc_client:main',
            'modbus_tcp_slave = scripts.modbus_tcp_slave:main',
            'tolk_server = scripts.tolk_server:main',
        ]
      })



""" Handler for dispatching incoming requests to
:class:`tolk.Dispatcher`.

"""
import socket
import errno
from logbook import Logger
from SocketServer import BaseRequestHandler

log = Logger(__name__)


class Handler(BaseRequestHandler):
    """ Handler for delegating incoming requests to :class:`Dispatcher`
    instance and return its response to client.

    :class:`Handler` is a subclass of :class:`SocketServer.BaseRequestHandler`
    and can be used in combination with a :class:`SocketServer.BaseServer`
    instance. This server must have an attribute :attr:`dispatcher` with an
    instance of :class:`Dispatcher`.

    Below you find a few lines of code to create a server listening on a Unix
    Domain Socket for JSON-RPC requests. These requests are delegated to a
    :class:`Dispatcher` instance which in turn sends Modbus request over RTU
    using a serial port::

        from modbus_tk.modbus_tcp import TcpMaster
        from SocketServer import UnixStreamServer

        from tolk import Dispatcher, Handler

        modbus_master = TcpMaster('localhost', 502)
        dispatcher = Dispatcher(modbus_master)

        server = UnixStreamServer('/tmp/tolk.sock', Handler)
        server.dispatcher = dispatcher

        server.serve_forever()

    """
    def handle(self):
        """ Direct incoming requests to server's dispatcher and return response
        back to client.
        """
        self.data = self.request.recv(1024).strip()
        log.debug('<-- {0}'.format(self.data))

        resp = self.server.dispatcher.call(self.data)
        log.debug('--> {0}'.format(resp))

        try:
            self.request.sendall(resp)
        except socket.error as e:

            # Catches broken pipe errors, errno 32. This is when client
            # terminates connection, but server still tries to send data to
            # client.
            if e.errno == errno.EPIPE:
                log.error('[Errno {0}]: Handler could not send response to '
                          'client.'.format(errno.EPIP))

                return
            raise
